/** KazgorColumn.java
 * Created on May 15, 2004
 *
 * Free.  Use however you see fit.
 */
package kazgorColumn;

import java.io.File;
import java.lang.String;




import org.gudy.azureus2.plugins.Plugin;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.tables.TableCell;
import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
import org.gudy.azureus2.plugins.ui.tables.TableColumn;
import org.gudy.azureus2.plugins.ui.tables.TableManager;
import org.gudy.azureus2.plugins.utils.Formatters;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;

/** Simple Column Example
 *
 * @author TuxPaper
 */
public class ExtraColumns implements Plugin {
  static final String COLUMNID_TORRENTNAME = "KazgorColumn1";
  static final String COLUMNID_UPETA = "KazgorColumn2";
  static final String COLUMNID_REMAINING = "KazgorColumn3";
  static final String COLUMNID_REMAINING_PERCENT = "KazgorColumn4";
  static final String COLUMNID_SDONE = "KazgorColumn5";
  
  // Copied from Constants - the plugin API should provide access to these
  // constants somewhere.
  static final String INFINITY_STRING = "\u221E";
  Formatters formatter;

  
  PluginInterface plugin_Interface;
  
  
   /** This method is called when the plugin is loaded / initialized
   *
   * @param pluginInterface access to Azureus' plugin interface
   */
    public void initialize(PluginInterface pluginInterface) {

        //set the global plugin_Interface to the local pluginInterface so later classes can use it
        plugin_Interface = pluginInterface;
        
        formatter = pluginInterface.getUtilities().getFormatters();
        TableManager tableManager = pluginInterface.getUIManager().getTableManager();
        TableColumn kazgorColumn;
    
        
        /* Torrent Filename - Add to the Complete(Seeding) column */    
        TorrentNameRefreshListener torrentNameRefreshListener = new TorrentNameRefreshListener();   
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE,COLUMNID_TORRENTNAME);
        kazgorColumn.initialize(TableColumn.ALIGN_LEAD, TableColumn.POSITION_LAST, 150);  
        kazgorColumn.addCellRefreshListener(torrentNameRefreshListener);
        tableManager.addColumn(kazgorColumn);
    
    
        /* Torrent Filename - Add to the Incomplete(downloading) column */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_INCOMPLETE, COLUMNID_TORRENTNAME);
        kazgorColumn.initialize(TableColumn.ALIGN_LEAD, TableColumn.POSITION_LAST, 150);
        kazgorColumn.addCellRefreshListener(torrentNameRefreshListener);
        tableManager.addColumn(kazgorColumn);
        
        
        /* Eta for Seeding column to get you to a given ratio */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE, COLUMNID_UPETA);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL,TableColumn.POSITION_LAST, 150, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new UploadEtaListener());
        tableManager.addColumn(kazgorColumn);
        
        /* Remaining for Seeding column to get you to a given ratio */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE, COLUMNID_REMAINING);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL,TableColumn.POSITION_LAST, 150, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new UploadRemainingListener());
        tableManager.addColumn(kazgorColumn);  
        
        /* Remaining % for Seeding column to get you to a given ratio */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_COMPLETE, COLUMNID_REMAINING_PERCENT);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL,TableColumn.POSITION_LAST, 80, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new UploadRemainingPercentListener());
        tableManager.addColumn(kazgorColumn);  

        /* selected files % done column */
        kazgorColumn = tableManager.createColumn(TableManager.TABLE_MYTORRENTS_INCOMPLETE, COLUMNID_SDONE);
        kazgorColumn.initialize(TableColumn.ALIGN_TRAIL, TableColumn.POSITION_LAST, 80, TableColumn.INTERVAL_LIVE);
        kazgorColumn.addCellRefreshListener(new SelectedDoneListener());
        tableManager.addColumn(kazgorColumn);

        //The magic to open a plugin GUI view -- see, wasn't that easy!
        //plugin_Interface.getUIManager().getSWTManager().addView((new View(pluginInterface)),false);
        
        //update opening the GUI to the new interface
        pluginInterface.getUIManager().addUIListener(
                new UIManagerListener()
                {
                  public void
                  UIAttached(
                          UIInstance instance )
                  {
                      if ( instance instanceof UISWTInstance ){
                          UISWTInstance swt = (UISWTInstance)instance;
                          View view = new View(plugin_Interface);
                          swt.addView(UISWTInstance.VIEW_MAIN, "ExtraColumns", view);
                      }
                  }

                 public void UIDetached(UIInstance arg0) {
                     
                     
                 }
                }); 
   }
  

/* Start of the Listener processes */  
    
   public class TorrentNameRefreshListener implements TableCellRefreshListener {
       public void refresh(TableCell cell) {
          Download dm = (Download)cell.getDataSource();
          if (dm == null)
            return;
                  
          String TorrentName = dm.getTorrentFileName();
     
    // Now lets just get the filename minus the path
          File fl = new File (TorrentName);
          TorrentName = fl.getName();
          
          String str = TorrentName;     
          int p = str.lastIndexOf(".torrent");
          
          //if the filename does not end in .torrent it must
          //be autoloaded from azureus itself (like my AZCVSUpdater does)
          //.. therefore it will end in .tmp instead
          if(p == -1){
              p = str.lastIndexOf(".tmp");
    //        if all else fails, just have the full name so you do not get errors
              if(p == -1){
                  p=str.length();
              }
              
          }
          
                
          TorrentName = str.substring(0, p);
      
          if (!cell.setSortValue(TorrentName) && cell.isValid())
            return;
               
          cell.setText(TorrentName);
        }
      }

   public class UploadEtaListener
		implements TableCellRefreshListener
	{

		public void refresh(TableCell cell) {
			
			Download dm = (Download) cell.getDataSource();
			if (dm == null)
				return;

			// For all none seeding torrent dont display anything in the coloum     
			int state = dm.getState();
			if (state != Download.ST_SEEDING) {
				if (!cell.setText("") && cell.isValid()) {
					// no cell text to update            
				} else {
					cell.setSortValue(-3);
				}
				return;
			}

			//long filesize = dm.getTorrent().getSize();
			long downloaded = dm.getStats().getDownloaded() - dm.getStats().getDiscarded() - dm.getStats().getHashFails() * dm.getTorrent().getPieceSize();
			long uploaded = dm.getStats().getUploaded();
			float ratio = getRatio();
			if (ratio < 0.2)
				ratio = 0.2f;

			float filesized_ratio = downloaded * ratio;

			if (uploaded >= filesized_ratio) {
				if (!cell.setText("Uploaded >= " + ratio + ":1") && cell.isValid()) {
					// no cell text to update
				} else {
					cell.setSortValue(-1);
				}
				return;
			}

			long upspeed = dm.getStats().getUploadAverage();

			if ((upspeed <= 0)) {
				if (!cell.setText(INFINITY_STRING) && cell.isValid()) {
					// no cell text to update
				} else {
					cell.setSortValue(-2);
				}
				return;
			}

			long eta_long =(long)(filesized_ratio - uploaded) / upspeed;

			if (!cell.setSortValue(eta_long) && cell.isValid())
				return;
			//-------------
			float remaining = 0;
			if (filesized_ratio >= uploaded) {
				remaining = filesized_ratio - uploaded;
			}

			float perc = ((remaining * 100) / filesized_ratio);
			try {
				if (perc >= 75) {
					cell.setForeground(
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color75_100_RED", 200),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color75_100_GREEN", 0),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color75_100_BLUE", 0));
				} else if ((perc >= 50) && (perc < 75)) {
					cell.setForeground(
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color50_75_RED", 255),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color50_75_GREEN", 128),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color50_75_BLUE", 64));
				} else if ((perc >= 25) && (perc < 50)) {
					cell.setForeground(
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color25_50_RED", 0),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color25_50_GREEN", 128),
							plugin_Interface.getPluginconfig().getPluginIntParameter(
									"color25_50_BLUE", 255));
				} else {
					if (perc != 0.0 && cell.isValid())
						cell.setForeground(
								plugin_Interface.getPluginconfig().getPluginIntParameter(
										"color0_25_RED", 64),
								plugin_Interface.getPluginconfig().getPluginIntParameter(
										"color0_25_GREEN", 0),
								plugin_Interface.getPluginconfig().getPluginIntParameter(
										"color0_25_BLUE", 128));
				}

			} catch (Exception e) {
				e.printStackTrace();
			}
			//--------------
			cell.setText(formatter.formatETAFromSeconds(eta_long));
		}
	}
   
   public class UploadRemainingListener implements TableCellRefreshListener {
    public void refresh(TableCell cell) {
      Download dm = (Download)cell.getDataSource();
      if (dm==null) return;

      float uploaded = dm.getStats().getUploaded();
      float real_download = dm.getStats().getDownloaded() - dm.getStats().getDiscarded() - dm.getStats().getHashFails() * dm.getTorrent().getPieceSize();

	  float ratio = getRatio();
      if (ratio < 0.2)
        ratio = 0.2f;

      float filesized_ratio = real_download * ratio;
      
      float remaining = 0;
      if (filesized_ratio > uploaded)
        remaining = filesized_ratio - uploaded;
      
      float perc = ((remaining * 100 ) / filesized_ratio);
      
      try{
          if (perc >= 75)
          {
			  cell.setForeground(
					  plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_RED", 200),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_BLUE",0)); 
           }
           else if ((perc >= 50) && (perc < 75)) 
           {
              cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_RED", 255),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_BLUE",64));
          } 
          else if ((perc >= 25) && (perc < 50)) 
          {
              cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_RED", 0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_BLUE",255));
          }
          else
          {
			  if(perc != 0.0 && cell.isValid())
				  cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_RED", 64),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_BLUE",128));
          }

          if (!cell.setSortValue((long) remaining) && cell.isValid())
			  return;
                  
          cell.setText(formatter.formatByteCountToKiBEtc((long) remaining));

		  String hovertext = "Upload Target ("+formatter.formatByteCountToKiBEtc((long) filesized_ratio)+")\n"; 
		  hovertext = hovertext + "Upload Ratio of ("+ratio+":1)";
				  
		  cell.setToolTip(hovertext);
				  
	  }catch (Exception e){
          //If a null point exception is still thrown (had it happen once!?!),
          //then ignore it
          e.printStackTrace();
      }
    }
   }
   
   public class UploadRemainingPercentListener implements TableCellRefreshListener {
    public void refresh(TableCell cell) {
      Download dm = (Download)cell.getDataSource();
      if (dm==null) return;
      
      float uploaded = dm.getStats().getUploaded();
	  float real_download = dm.getStats().getDownloaded() - dm.getStats().getDiscarded() - dm.getStats().getHashFails() * dm.getTorrent().getPieceSize();
	  
	  float ratio = getRatio();
      if (ratio < 0.2)
        ratio = 0.2f;
      
      float filesized_ratio = real_download * ratio;
      
      float remaining = 0;
      if (filesized_ratio >= uploaded)
         remaining = filesized_ratio - uploaded;
      
      float perc = ((int)(remaining * 10000 / filesized_ratio)) / 100.f; //floor 2digit precision
      
      try{
          if (perc >= 75)
          {
			  cell.setForeground(
					  plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_RED", 200),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color75_100_BLUE",0)); 
           }
           else if ((perc >= 50) && (perc < 75)) 
           {
              cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_RED", 255),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color50_75_BLUE",64));
          } 
          else if ((perc >= 25) && (perc < 50)) 
          {
              cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_RED", 0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_GREEN",128),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color25_50_BLUE",255));
          }
          else
          {
			  if(perc != 0.0 && cell.isValid())
				  cell.setForeground(
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_RED", 64),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_GREEN",0),
                      plugin_Interface.getPluginconfig().getPluginIntParameter("color0_25_BLUE",128));
          }
         
          if (!cell.setSortValue((long) remaining) && cell.isValid())
			  return;

          cell.setText(""+perc+"%");

		  String hovertext = "Upload Remaining ("+formatter.formatByteCountToKiBEtc((long) remaining)+")\n";
		  hovertext += "Upload Target ("+formatter.formatByteCountToKiBEtc((long) filesized_ratio)+")\n"; 
		  hovertext += "Upload Ratio of ("+ratio+":1)";
		  //hovertext += "\ndiscarded ("+formatter.formatByteCountToKiBEtc(dm.getStats().getDiscarded())+")";
		  //hovertext += "\nhash fails ("+formatter.formatByteCountToKiBEtc(dm.getStats().getHashFails())+")";
				  
		  cell.setToolTip(hovertext);
				  
	  }catch (Exception e){
          //If a null point exception is still thrown (had it happen once!?!),
          //then ignore it
          e.printStackTrace();
      }
    }
   }

   public class SelectedDoneListener implements TableCellRefreshListener {
	    public void refresh(TableCell cell) {
	      Download dm = (Download)cell.getDataSource();
	      if (dm==null) return;

	      long filessize = getDownloadingFilesTotalSize(dm.getDiskManagerFileInfo());
          long downloaded = filessize - dm.getStats().getRemaining(); 
          
          if (!cell.setSortValue(downloaded) && cell.isValid())  
        	  return;
          
          cell.setSortValue(downloaded);
          
          float perc = (downloaded * 10000 / filessize) / 100.f; //floor 2digit precision
          
          if (perc < 0)
        	  cell.setText("0.00%");
          else
        	  cell.setText(""+perc+"%");
          
		  String hovertext = "Files Selected Total Size: "+formatter.formatByteCountToKiBEtc(filessize); 
		  cell.setToolTip(hovertext);
	    }
   }

   /** 
    * get byte size of all files marked for downloading of one torrent
    */
   public long getDownloadingFilesTotalSize(DiskManagerFileInfo[] dmfi) {
	   long tsize = 0;
	   for (int i=0; i<dmfi.length; i++)
		   if (!dmfi[i].isSkipped() && !dmfi[i].isDeleted())
			   tsize += dmfi[i].getLength();
	   return tsize;
   }
   /**
    * Method to get the ratio from the user settings
    * @return float
    */
   public float getRatio(){
       try
	   {
           switch (plugin_Interface.getPluginconfig().getPluginIntParameter("ratio_type",2))
           {
           case 1:
			   return plugin_Interface.getPluginconfig().getUnsafeIntParameter("StartStopManager_iFirstPriority_ShareRatio",1000) / 1000.f;
           case 2:
               return plugin_Interface.getPluginconfig().getUnsafeFloatParameter("Stop Ratio"); 
           default:
			   return getManualRatio();
           }
       } 
	   catch (Exception e)
	   {
           e.printStackTrace();
           return 1.f;
       }
       
   }
 
   /**
    * Method to get the Manual set Ration out of a String stored value 
    * @return float ratio
    */
   public float getManualRatio(){
       try{
           return Float.valueOf(plugin_Interface.getPluginconfig().getPluginStringParameter("manual_ratio","2")).floatValue();
       }catch(Exception e){
           e.printStackTrace();
           return 2;
       }
       
   }

}//EOF
